# coding: utf-8
import logging
import os

import tweepy

# https://github.com/tweepy/tweepy
# https://dev.twitter.com/docs
# https://dev.twitter.com/apps/ID/show


def tweet(message, consumer_key=None, consumer_secret=None, access_token=None, access_token_secret=None, debug=False):
    consumer_key = os.environ.get('TWITTER_CONSUMER_KEY', consumer_key)
    consumer_secret = os.environ.get('TWITTER_CONSUMER_SECRET', consumer_secret)
    access_token = os.environ.get('TWITTER_ACCESS_TOKEN', access_token)
    access_token_secret = os.environ.get('TWITTER_ACCESS_TOKEN_SECRET', access_token_secret)
    if not debug:
        try:
            #authenticator = tweepy.auth.BasicAuthHandler(TWITTER_USERNAME, TWITTER_PASSWORD)
            authenticator = tweepy.auth.OAuthHandler(consumer_key, consumer_secret)
            authenticator.set_access_token(access_token, access_token_secret)

            api = tweepy.API(authenticator)

            api.update_status(message[0:140])
        except Exception as e:
            logging.error('%s' % str(e))
            logging.error('Fail to tweet: %s' % message)
            logging.exception(e)
            # We do not want to interrupt the application because a tweet error.



import json
import os
import unittest

from ..facebook_services import *


class FacebookUtilsTests(unittest.TestCase):
    def test_get_event_id_from_facebook_url(self):
        self.assertEquals(123, get_event_id_from_facebook_url('http://facebook.com/events/123'))
        self.assertEquals(456, get_event_id_from_facebook_url('http://facebook.com/events/456?x=1'))
        self.assertEquals(None, get_event_id_from_facebook_url('123'))
        self.assertEquals(None, get_event_id_from_facebook_url('abc'))

    def test_get_user_id_from_facebook_url(self):
        self.assertEquals(123, get_user_id_from_facebook_url('https://www.facebook.com/profile.php?id=123'))
        self.assertEquals(456, get_user_id_from_facebook_url('https://www.facebook.com/profile.php?id=456&fref=ts'))
        self.assertEquals(None, get_user_id_from_facebook_url('123'))
        self.assertEquals(None, get_user_id_from_facebook_url('abc'))


class FacebookTestCase(unittest.TestCase):
    FB_CLASS = FacebookGraphApi
    FACEBOOK_JSON = """
    {}
    """

    def setUp(self):
        self.fb = self.FB_CLASS(123)
        self.fb.load_graph = lambda url: json.loads(self.FACEBOOK_JSON)
        self.fb.load()


class FacebookUserTests(FacebookTestCase):
    FB_CLASS = FacebookUser
    FACEBOOK_JSON = """
    {
        "id": "123",
        "username": "paulocheque"
    }
    """

    def test_get_field(self):
        self.assertEquals('123', self.fb.get_field('id'))
        self.assertEquals('paulocheque', self.fb.get_field('username'))
        self.assertEquals('paulocheque@facebook.com', self.fb.get_field('email'))


class FacebookCommunityTests(FacebookTestCase):
    FB_CLASS = FacebookCommunity
    FACEBOOK_JSON_FEED = """
    {
        "data": [
            {
                "id":123,
                "message": "http://facebook.com/events/123",
                "likes": {
                    "data": [
                        {"id":1},{"id":2}
                    ]
                },
                "comments": {
                    "data": [
                        {"from": {"id":1}},
                        {"from": {"id":3}}
                    ]
                }
            },
            {
                "id":456,
                "link": "http://facebook.com/events/456",
                "likes": {
                    "data": [
                        {"id":1},{"id":2}
                    ]
                },
                "comments": {
                    "data": [
                        {"from": {"id":1}},
                        {"from": {"id":3}}
                    ]
                }
            }
        ],
        "paging": {
          "next": "http://next-url",
          "previous": "http://previous-url"
        }
    }
    """
    FACEBOOK_JSON2 = """
    {
       "data": [

       ]
    }
    """

    def setUp(self):
        super(FacebookCommunityTests, self).setUp()
        self.fb.load_graph = lambda url: json.loads(self.FACEBOOK_JSON_FEED)
        self.fb.load_feed()

    def test_get_events_ids_from_feed(self):
        fb_ids = self.fb.get_events_ids_from_feed()
        self.assertEquals(2, len(fb_ids))
        self.assertEquals(123, fb_ids[1])
        self.assertEquals(456, fb_ids[0])

    def test_get_recent_users_ids_from_feed(self):
        fb_ids = self.fb.get_recent_users_ids_from_feed()
        self.assertEquals(3, len(fb_ids))

    def test_get_all_users_ids(self):
        fb_ids = self.fb.get_all_users_ids()
        self.assertEquals(3, len(fb_ids))

    # def test_get_all_users(self):
    #     self.fb.get_all_users(fields='id')

    def test_has_feed_to_load(self):
        self.fb.load_graph = lambda url: json.loads(self.FACEBOOK_JSON2)
        self.assertEquals(True, self.fb.has_feed_to_load())
        self.fb.load_feed()
        self.assertEquals(False, self.fb.has_feed_to_load())


class FacebookEventTests(FacebookTestCase):
    FB_CLASS = FacebookEvent
    FACEBOOK_JSON = """
    {
       "id": "EVENT ID",
       "owner": {
          "name": "USER NAME",
          "id": "USER ID"
       },
       "name": "NAME",
       "description": "EVENT DESCRIPTION",
       "start_time": "2010-03-14T14:00:00",
       "end_time": "2010-03-14T17:30:00",
       "location": "EVENT LOCATION",
       "venue": {
          "street": "409 Colorado St.",
          "city": "Austin",
          "state": "Texas",
          "country": "United States",
          "latitude": 30.2669,
          "longitude": -97.7428
       },
       "privacy": "OPEN",
       "updated_time": "2010-04-13T15:29:40+0000"
    }
    """
    FACEBOOK_JSON_MAYBE_ATTENDING = """
    {
        "data": [
            {"id":123},
            {"id":456}
        ]
    }
    """

    def test_get_name(self):
        self.assertEquals('NAME', self.fb.get_name())

    def test_get_location(self):
        self.assertEquals('EVENT LOCATION', self.fb.get_location())

    def test_get_date_str(self):
        self.assertEquals('2010-03-14', self.fb.get_date_str())

    def test_get_time_str(self):
        self.assertEquals('14:00', self.fb.get_time_str())

    def test_get_lat_long(self):
        self.assertEquals((30.2669, -97.7428), self.fb.get_lat_long())

    def test_maybe(self):
        self.fb.load_graph = lambda url: json.loads(self.FACEBOOK_JSON_MAYBE_ATTENDING)
        self.fb.load_maybe()
        fb_ids = self.fb.get_recent_users_ids_from_maybe()
        self.assertEquals(2, len(fb_ids))

    def test_attending(self):
        self.fb.load_graph = lambda url: json.loads(self.FACEBOOK_JSON_MAYBE_ATTENDING)
        self.fb.load_attending()
        fb_ids = self.fb.get_recent_users_ids_from_attending()
        self.assertEquals(2, len(fb_ids))


class FacebookGroupTests(FacebookTestCase):
    FB_CLASS = FacebookGroup
    FACEBOOK_JSON = """
    {
       "email": "group@groups.facebook.com",
       "privacy": "OPEN"
    }
    """
    FACEBOOK_JSON_MEMBERS = """
    {
       "data": [
          {
             "name": "PySocial1",
             "administrator": true,
             "id": "123"
          },
          {
             "name": "PySocial2",
             "administrator": true,
             "id": "456"
          }
       ],
       "paging": {
          "next": "https://graph.facebook.com/123/members?limit=100&offset=100"
       }
    }
    """

    def test_email(self):
        self.assertEquals('group@groups.facebook.com', self.fb.get_email())

    def test_get_privacy(self):
        self.assertEquals('OPEN', self.fb.get_privacy())

    def test_get_recent_users_ids_from_members(self):
        self.fb.load_graph = lambda url: json.loads(self.FACEBOOK_JSON_MEMBERS)
        self.fb.load_members()
        fb_ids = self.fb.get_recent_users_ids_from_members()


class FacebookPageTests(FacebookTestCase):
    FB_CLASS = FacebookPage
    FACEBOOK_JSON = """
    {
       "data": [

       ]
    }
    """



# coding: utf-8
from datetime import datetime, timedelta
import logging
import os
import re
import sys

from dateutil.parser import *
import requests


class FacebookError(Exception): pass

class FacebookConnectionError(FacebookError): pass

class FacebookConnectionLimit(FacebookError): pass

class FacebookDataError(FacebookError): pass

class FacebookPermissionError(FacebookError): pass

class FacebookInvalidId(FacebookError): pass


def get_event_id_from_facebook_url(url):
    value = re.match(r'.+/events/(?P<fb_id>\d+).*', url)
    if value:
        return int(value.group('fb_id'))
    return None

def get_user_id_from_facebook_url(url):
    value = re.match(r'.+id=(?P<fb_id>\d+).*', url)
    if value:
        return int(value.group('fb_id'))
    return None


# http://developers.facebook.com/tools/explorer/
class FacebookGraphApi(object):
    URL = 'https://graph.facebook.com/%(fb_id)s/'
    # http://developers.facebook.com/docs/authentication/
    URL_TOKEN = 'https://graph.facebook.com/oauth/access_token?grant_type=client_credentials&client_id=%(app_id)s&client_secret=%(app_secret)s'

    def __init__(self, fb_id, app_id=None, app_secret=None):
        self.data = {}
        self.fb_id = fb_id
        self.app_id = app_id if app_id else os.environ.get('FACEBOOK_API_KEY', '')
        self.app_secret = app_secret if app_secret else os.environ.get('FACEBOOK_API_SECRET', '')

        self._access_token = None
        self._number_of_requests = 0
        self._timestamp_token_updated = None

    def set_log_level(self, logging_level):
        logging.getLogger().setLevel(logging_level)

    # http://developers.facebook.com/blog/post/2011/05/13/how-to--handle-expired-access-tokens/
    def update_access_token(self):
        token_not_loaded = self._access_token is None
        token_expired = (not self._timestamp_token_updated) or datetime.now() > self._timestamp_token_updated + timedelta(minutes=5)
        too_many_requests = self._number_of_requests % 50 == 0
        if  too_many_requests or token_expired or token_not_loaded:
            url = self.URL_TOKEN % dict(app_id=self.app_id, app_secret=self.app_secret)
            self._access_token = requests.get(url).text
            self._timestamp_token_updated = datetime.now()
        return self._access_token

    def _add_access_token_to_url(self, graph_url):
        if '?' in graph_url:
            return ''.join([graph_url, '&', self._access_token])
        else:
            return ''.join([graph_url, '?', self._access_token])

    def validate_status_code(self, code):
        if code == 401 or code == 403:
            raise FacebookPermissionError('Invalid access token for this action')
        elif code == 404:
            raise FacebookInvalidId('Is the url correct? %s' % self.fb_id)
        elif code == 400: # py-social bug or bad usage of the lib or country/age/etc restriction
            raise FacebookPermissionError('Is your access token valid for this action?')

    def validate_response(self, data):
        error = data.get('error', None)
        if error:
            # {
            #    "error": {
            #       "message": "An access token is required to request this resource.",
            #       "type": "OAuthException",
            #       "code": 104
            #    }
            # }
            code = int(error.get('code', 0))
            msg = error.get('message', '')
            error_type = error.get('type', '')
            if code == 104 or code == 102 or error_type == 'OAuthException':
                raise FacebookPermissionError('Invalid access token for this action: ' + msg)
            elif code == 17 or code == 4 or code == 613:
                # https://developers.facebook.com/docs/reference/ads-api/api-rate-limiting/
                raise FacebookConnectionLimit('Too many requests to Graph API: ' + msg)
            else:
                raise FacebookError(msg)

    def load_graph(self, graph_url):
        self._number_of_requests += 1
        graph_url = graph_url % dict(fb_id=self.fb_id)
        self.update_access_token()
        graph_url = self._add_access_token_to_url(graph_url)
        logging.info(graph_url)
        headers = {'content-type': 'application/json'}
        r = requests.get(graph_url, headers=headers)
        logging.debug(r)
        self.validate_status_code(r.status_code)
        self.validate_response(r.json())
        return r.json()

    def load(self, **kwargs):
        params = '&'.join([k + '=' + v for k, v in kwargs.items()])
        self.data = self.load_graph(self.URL + '?' + params)
        return self.data

    def load_image(self, graph_url):
        self._number_of_requests += 1
        self.update_access_token()
        graph_url = self._add_access_token_to_url(graph_url)
        logging.info(graph_url)
        r = requests.get(graph_url)
        return r.content

    def parse_timestamp(self, timestamp_str):
        # https://developers.facebook.com/docs/reference/api/dates/
        # Facebook default: ISO8601
        # e.g. 2012-12-15T14:00:02+0000
        return parse(timestamp_str) if timestamp_str else timestamp_str

    def parse_in_naive_timestamp(self, timestamp_str):
        if not timestamp_str:
            return timestamp_str
        t = self.parse_timestamp(timestamp_str)
        format = '%Y-%m-%d %H:%M'
        s = t.strftime(format)
        return datetime.strptime(s, format)

    def format_email(self, email):
        if email:
            return email.replace('\u0040', '@')
        return email


class FacebookUser(FacebookGraphApi):
    "https://developers.facebook.com/docs/reference/api/user/"
    URL = r'https://graph.facebook.com/%(fb_id)s'

    def load(self, fields='email,gender,languages,username,verified', **kwargs):
        return super(FacebookUser, self).load(**kwargs)

    def get_field(self, field):
        if field == 'email':
            return self.get_email()
        return self.data.get(field, None)

    def get_email(self):
        email = self.data.get('email', None)
        username = self.data.get('username', None)
        if not email and username:
            email = '%s@facebook.com' % username
        return self.format_email(email)


class FacebookCommunity(FacebookGraphApi):
    URL_FEED = 'https://graph.facebook.com/%(fb_id)s/feed'

    def __init__(self, fb_id, app_id=None, app_secret=None):
        super(FacebookCommunity, self).__init__(fb_id, app_id=app_id, app_secret=app_secret)
        self.feed = []
        self.next_feed = self.URL_FEED

    def load_feed(self, pages=1):
        if self.has_feed_to_load():
            result = self.load_graph(self.next_feed)
            self.feed += result.get('data', [])
            self.next_feed = result.get('paging', {}).get('next', None)
            if pages > 0:
                self.load_feed(pages=pages-1)
        return self.feed

    def has_feed_to_load(self):
        return self.next_feed is not None

    def get_events_ids_from_feed(self):
        event_ids = []
        for f in self.feed:
            url_re_expression = '.+facebook.com/events/(?P<event_id>\d+).*'
            if 'message' in f:
                try:
                    event_id = re.search(url_re_expression, f['message']).group(1)
                    event_ids.append(int(event_id))
                except AttributeError:
                    pass # not a event link
            if 'link' in f:
                try:
                    event_id = re.search(url_re_expression, f['link']).group(1)
                    event_ids.append(int(event_id))
                except AttributeError:
                    pass # not a event link
        return list(set(event_ids)) # remove replicated links

    def get_recent_users_ids_from_feed(self):
        fb_ids = []
        for f in self.feed:
            if f.get('likes', None):
                fb_ids_from_likes = [l['id'] for l in f['likes']['data']]
                fb_ids += fb_ids_from_likes
            if f.get('comments', None):
                fb_ids_from_comments = [u['from']['id'] for u in f['comments']['data']]
                fb_ids += fb_ids_from_comments
        return list(set(fb_ids))

    def get_all_users_ids(self):
        users = []
        feed_users = self.get_recent_users_ids_from_feed()
        users += feed_users
        return list(set(users))

    def get_all_users(self, fields='email,username', offset=0, limit=5000):
        users = []
        for user_id in self.get_all_users_ids()[offset:limit]:
            fb = FacebookUser(user_id, app_id=self.app_id, app_secret=self.app_secret)
            fb.load(fields=fields)
            fields_splitted = fields.split(',')
            user = dict(fb_id=user_id)
            for f in fields_splitted:
                user[f] = fb.get_field(f)
            users.append(user)
        return users

    def _get_users_ids_from_list(self, result):
        return [u['id'] for u in result]


class FacebookPage(FacebookCommunity):
    "http://developers.facebook.com/docs/reference/api/page/"


class FacebookEvent(FacebookCommunity):
    "https://developers.facebook.com/docs/graph-api/reference/event"
    URL_MAYBE = 'https://graph.facebook.com/%(fb_id)s/maybe'
    URL_ATTENDING = 'https://graph.facebook.com/%(fb_id)s/attending'
    URL_PICTURE = r'https://graph.facebook.com/%(fb_id)s/picture?type=small'
    #http://developers.facebook.com/docs/reference/fql/event/
    URL_ALL_PICTURES = r'https://graph.facebook.com/fql?q=SELECT+pic,pic_big,pic_small+FROM+event+WHERE+eid=%(fb_id)s'

    def __init__(self, fb_id, app_id=None, app_secret=None):
        super(FacebookEvent, self).__init__(fb_id, app_id=app_id, app_secret=app_secret)
        self.maybe = {}
        self.attending = {}
        self.flyer = None
        self.flyers_info = None
        self.flyer_default = None
        self.flyer_big = None
        self.flyer_small = None

    def load_maybe(self):
        self.maybe = self.load_graph(self.URL_MAYBE)
        return self.maybe

    def load_attending(self):
        self.attending = self.load_graph(self.URL_ATTENDING)
        return self.attending

    def load_small_flyer(self):
        self.flyer = self.load_image(self.URL_PICTURE)
        return self.flyer

    def load_flyers(self):
        self.flyers_info = self.load_graph(self.URL_ALL_PICTURES)
        try:
            self.flyer_default = self.load_image(self.flyers_info['data'][0]['pic'])
        except (KeyError, IndexError):
            pass
        try:
            self.flyer_big = self.load_image(self.flyers_info['data'][0]['pic_big'])
        except (KeyError, IndexError):
            pass
        try:
            self.flyer_small = self.load_image(self.flyers_info['data'][0]['pic_small'])
        except (KeyError, IndexError):
            pass
        return self.flyers_info

    def get_recent_users_ids_from_maybe(self):
        return list(set(self._get_users_ids_from_list(self.maybe.get('data', []))))

    def get_recent_users_ids_from_attending(self):
        return list(set(self._get_users_ids_from_list(self.attending.get('data', []))))

    def get_all_users_ids(self):
        users = super(FacebookEvent, self).get_all_users_ids()
        users += self.get_recent_users_ids_from_maybe()
        users += self.get_recent_users_ids_from_attending()
        return list(set(users))

    def get_name(self):
        return self.data.get('name', None)

    def get_owner(self):
        return self.data.get('owner', {}).get('id', None)

    def get_lat_long(self):
        try:
            return self.data['venue']['latitude'], self.data['venue']['longitude']
        except (KeyError, TypeError):
            # sometimes the FB returns a {} and other times a [], if venue is empty.
            return None, None

    def get_location(self):
        return self.data.get('location', '')

    def get_timestamp(self):
        return self.parse_timestamp(self.data.get('start_time', None))

    def get_naive_timestamp(self):
        return self.parse_in_naive_timestamp(self.data.get('start_time', None))

    def get_timestamp_str(self):
        return self.get_timestamp().strftime('%Y-%m-%d %H:%M')

    def get_date_str(self):
        return self.get_timestamp().strftime('%Y-%m-%d')

    def get_time_str(self):
        # http://developers.facebook.com/docs/reference/api/
        return self.get_timestamp().strftime('%H:%M')

    def get_flyer(self):
        return self.flyer

    def get_flyer_default(self):
        return self.flyer_default

    def get_flyer_big(self):
        return self.flyer_big

    def get_flyer_small(self):
        return self.flyer_small


class FacebookGroup(FacebookCommunity):
    "http://developers.facebook.com/docs/reference/api/group/"
    URL_MEMBERS = 'https://graph.facebook.com/%(fb_id)s/members'

    def __init__(self, fb_id, app_id=None, app_secret=None):
        super(FacebookGroup, self).__init__(fb_id, app_id=app_id, app_secret=app_secret)
        self.members = {}

    def load_members(self, offset=0, limit=1000): # TODO: offset+limit
        self.members = self.load_graph(self.URL_MEMBERS)
        return self.members

    def get_recent_users_ids_from_members(self):
        return list(set(self._get_users_ids_from_list(self.members.get('data', []))))

    def get_all_users_ids(self):
        users = super(FacebookGroup, self).get_all_users_ids()
        users += self.get_recent_users_ids_from_members()
        return list(set(users))

    def get_email(self):
        "Send e-mail to post in Group's feed"
        return self.format_email(self.data.get('email', None))

    def get_privacy(self):
        return self.data.get('privacy', None)



from datetime import datetime
import os
import unittest

from ..facebook_services import *

# export TEST_FB_USER=1392965554
# export TEST_FB_EVENT=679999435376343
# export TEST_FB_PAGE=627676453946073
# export TEST_FB_GROUP=
# export TEST_FB_APP_ID=
# export TEST_FB_APP_SECRET=
class FacebookTestCase(unittest.TestCase):
    user_id = os.getenv('TEST_FB_USER')
    event_id = os.getenv('TEST_FB_EVENT')
    page_id = os.getenv('TEST_FB_PAGE')
    group_id = os.getenv('TEST_FB_GROUP')
    app_id = os.getenv('TEST_FB_APP_ID')
    app_secret = os.getenv('TEST_FB_APP_SECRET')
    FB_CLASS = FacebookGraphApi
    fb_id = ''

    def setUp(self):
        super(FacebookTestCase, self).setUp()
        self.fb = self.FB_CLASS(self.fb_id, app_id=self.app_id, app_secret=self.app_secret)
        self.fb.load()


class FacebookUserInfoTests(FacebookTestCase):
    FB_CLASS = FacebookUser
    fb_id = FacebookTestCase.user_id

    def test_get_info(self):
        self.assertEquals('1392965554', self.fb.get_field('id'))
        self.assertEquals('male', self.fb.get_field('gender'))
        self.assertEquals('paulocheque@gmail.com', self.fb.get_field('email'))


class FacebookCommunityTests(FacebookTestCase):
    FB_CLASS = FacebookCommunity
    fb_id = FacebookTestCase.group_id # to avoid errors of empty id

    def setUp(self):
        super(FacebookCommunityTests, self).setUp()
        self.fb.load_feed()

    def test_get_events_ids_from_feed(self):
        fb_ids = self.fb.get_events_ids_from_feed()
        self.assertEquals(True, len(fb_ids) >= 0)

    def test_get_recent_users_ids_from_feed(self):
        fb_ids = self.fb.get_recent_users_ids_from_feed()
        self.assertEquals(True, len(fb_ids) >= 0)

    def test_get_all_users_ids(self):
        fb_ids = self.fb.get_all_users_ids()
        self.assertEquals(True, len(fb_ids) >= 0)

    def test_get_all_users(self):
        users = self.fb.get_all_users(limit=1)
        self.assertEquals(True, len(users) >= 0)


class FacebookEventTests(FacebookCommunityTests):
    FB_CLASS = FacebookEvent
    fb_id = FacebookTestCase.event_id

    def test_get_name(self):
        r = u"Teste"
        self.assertEquals(r, self.fb.get_name())

    def test_get_location(self):
        self.assertEquals(u'FMRP USP', self.fb.get_location())

    def test_get_timestamp(self):
        self.assertEquals('2014-03-06 00:00', self.fb.get_timestamp().strftime('%Y-%m-%d %H:%M'))

    def test_get_naive_timestamp(self):
        self.assertEquals(datetime(2014, 03, 06, 00, 00, 00), self.fb.get_naive_timestamp())

    def test_get_date_str(self):
        self.assertEquals('2014-03-06', self.fb.get_date_str())

    def test_get_time_str(self):
        self.assertEquals('00:00', self.fb.get_time_str())

    def test_maybe(self):
        self.fb.load_maybe()
        self.fb.get_recent_users_ids_from_maybe()

    def test_attending(self):
        self.fb.load_attending()
        self.fb.get_recent_users_ids_from_attending()

    def test_small_flyer(self):
        self.fb.load_small_flyer()
        self.fb.get_flyer()

    def test_flyers(self):
        self.fb.load_flyers()
        self.fb.get_flyer_default()
        self.fb.get_flyer_big()
        self.fb.get_flyer_small()


class FacebookGroupTests(FacebookCommunityTests):
    FB_CLASS = FacebookGroup
    fb_id = FacebookTestCase.group_id

    def test_email(self):
        self.assertEquals('606388089437098@groups.facebook.com', self.fb.get_email())

    def test_get_privacy(self):
        self.assertEquals('OPEN', self.fb.get_privacy())

    def test_get_recent_users_ids_from_members(self):
        self.fb.load_members()
        fb_ids = self.fb.get_recent_users_ids_from_members()


class FacebookPageTests(FacebookCommunityTests):
    FB_CLASS = FacebookPage
    fb_id = FacebookTestCase.page_id




# coding: utf-8
# Selenium script to post message in Facebook
import logging
import os
import re

from selenium import webdriver


class FacebookSpider(object):
    def __init__(self, email, password):
        self.email = email
        self.password = password
        self.driver = None

    def start(self):
        # https://code.google.com/p/selenium/wiki/PythonBindings
        # Depends on: java - jar selenium-server.jar
        # self.driver = webdriver.Remote(desired_capabilities=webdriver.DesiredCapabilities.HTMLUNITWITHJS)
        self.driver = webdriver.Chrome()
        self.driver.get('http://facebook.com')

    def authenticate(self):
        email = self.driver.find_element_by_id('email')
        email.send_keys(self.email)
        password = self.driver.find_element_by_id('pass')
        password.send_keys(self.password)
        login_button = self.driver.find_elements_by_xpath("//input[@type='submit' and @value='Log In']")[0]
        login_button.click()

    def send_message(self, facebook_id, message):
        try:
            print('Message to %s' % (facebook_id))
            self.driver.get('http://facebook.com/messages/%s' % facebook_id)

            if message in self.driver.page_source:
                print(u'This message has already been sent to the user: %s' % facebook_id)
                return False

            text_area = self.driver.find_elements_by_xpath("//textarea[@name='message_body']")[0]
            text_area.send_keys(message)

            try:
                send_button = self.driver.find_elements_by_xpath("//input[@type='submit' and @value='Send']")[0]
            except IndexError:
                send_button = self.driver.find_elements_by_xpath("//input[@type='submit' and @value='Reply']")[0]
            send_button.click()
            print('OK: %s' % facebook_id)
        except Exception as e:
            logging.exception(e)
            print(repr(e))
            print('FAIL: %s' % facebook_id)
            # print(self.driver.page_source)
            return False
        return True

    def send_messages(self, facebook_ids, message):
        succeeded_ids = []
        for facebook_id in facebook_ids:
            if self.send_message(facebook_id, message):
                succeeded_ids.append(facebook_id)
        return succeeded_ids

    def close(self):
        if self.driver:
            self.driver.quit()


def send_facebook_messages(facebook_ids, message, email=None, password=None):
    if facebook_ids:
        email = email if email else os.getenv('FB_EMAIL')
        password = password if password else os.getenv('FB_PASS')
        spider = FacebookSpider(email, password)
        try:
            spider.start()
            spider.authenticate()
            succeeded_ids = spider.send_messages(facebook_ids, message)
            return succeeded_ids
        finally:
            spider.close()
    else:
        print('No Facebook id to send message')
    return []


def colorize(message, color='blue'):
    color_codes = dict(black=30, red=31, green=32, yellow=33, blue=34, magenta=35, cyan=36, white=37)
    code = color_codes.get(color, 34)
    msg = '\033[%(code)sm%(message)s\033[0m' % {'code':code, 'message':message}
    # print(msg)
    return msg


# python scripts/send_facebook_message.py facebook_msg.txt facebook_ids.txt USER PASSWORD
if __name__ == '__main__':
    import os
    import sys
    import argparse
    import logging

    try:
        pass
    except ImportError as e:
        print(os.getcwd())
        print(sys.path)
        raise e

    BEER = '\xF0\x9f\x8d\xba '
    VERSION = 1
    print(colorize('%s - Facebook Spider (%s)' % (BEER, VERSION), color='yellow'))

    parser = argparse.ArgumentParser()
    parser.add_argument('msg', help='Filename with the message to be sent')
    parser.add_argument('fb_ids', help='Filename with Facebook ids separated by commas')
    parser.add_argument('fb_email', help='Facebook email')
    parser.add_argument('fb_pass', help='Facebook password')
    args = parser.parse_args()

    print(colorize('::: Sending message', color='blue'))

    f = open(args.msg, 'r')
    message = f.read().decode('utf-8')
    f.close()

    f = open(args.fb_ids, 'r')
    facebook_ids = f.read().decode('utf-8').strip().split(',')
    facebook_ids = filter(str, facebook_ids)
    f.close()

    email = args.fb_email
    password = args.fb_pass
    send_facebook_messages(facebook_ids, message, email, password)



# coding: utf-8
import base64
import logging
import sys
import urllib2


def request(url, content_type, timeout=240, username=None, password=None, host=None):
    opener = urllib2.build_opener(urllib2.HTTPHandler)
    request = urllib2.Request(url)
    if host:
        request.add_header('Host', host)
    request.add_header('Accept', '*/*')
    request.add_header('Content-type', content_type)
    if username:
        base64string = base64.encodestring('%s:%s' % (username, password)).replace('\n', '')
        request.add_header("Authorization", "Basic %s" % base64string)
    #logging.info('HTTP Headers: %s' % request.headers)
    try:
        handler = opener.open(request, timeout=timeout)
        return handler.read()
    except urllib2.HTTPError as e:
        logging.error('HttpError %s: %s [%s]' % (e.code, repr(e), url))
        raise e, None, sys.exc_info()[2]
    except IOError as e:
        logging.error('IOError: %s' % repr(e))
        raise e, None, sys.exc_info()[2]



#from distutils.core import setup
from setuptools import setup, find_packages

VERSION = '0.1.7'
PROJECT_NAME = 'py-social'

tests_require = [
    'nose',
    'coverage',
]

install_requires = [
    'six',
    'requests',
    'python-dateutil',
    'simplejson',
    'tweepy',
    'facebook-sdk',
    'selenium',
]

setup(name='%s' % PROJECT_NAME,
      url='https://github.com/paulocheque/%s' % PROJECT_NAME,
      author="paulocheque",
      author_email='paulocheque@gmail.com',
      keywords='',
      description='',
      license='MIT',
      classifiers=[
          # 'Framework :: Tornado',
          'Operating System :: OS Independent',
          'Topic :: Software Development'
      ],

      version='%s' % VERSION,
      install_requires=install_requires,
      tests_require=tests_require,
      # test_suite='runtests.runtests',
      # extras_require={'test': tests_require},

      packages=find_packages(),
)

